@@ -122,10 +122,12 @@
 #' Furthermore an \code{RLum.Results} object is returned with the following structure:\cr
 #'
 #' data:\cr
+#' .. $data : \code{data.frame} with fitting results\cr
 #' .. $fit : \code{nls} (nls object)\cr
-#' .. $output.table : \code{data.frame} with fitting results\cr
 #' .. $component.contribution.matrix : \code{list} component distribution matrix\cr
-#' .. $call : \code{call} the original function call
+#'
+#' info:\cr
+#' .. $call : \code{call} the original function call\cr
 #'
 #' Matrix structure for the distribution matrix:\cr
 #'
@@ -143,7 +145,7 @@
 #' global minimum rather than a local minimum! In any case of doubt, the use of
 #' manual start values is highly recommended.
 #'
-#' @section Function version: 0.3.1
+#' @section Function version: 0.3.2
 #'
 #' @author Sebastian Kreutzer, IRAMAT-CRP2A, Universite Bordeaux Montaigne
 #' (France)
@@ -998,15 +1000,14 @@ fit_LMCurve<- function(
   ##============================================================================#
   ## Return Values
   ##============================================================================#
-
   newRLumResults.fit_LMCurve <- set_RLum(
     class = "RLum.Results",
     data = list(
+      data = output.table,
       fit = fit,
-      output.table = output.table,
-      component.contribution.matrix = list(component.contribution.matrix),
-      call = sys.call()
-    )
+      component.contribution.matrix = list(component.contribution.matrix)
+    ),
+    info = list(call = sys.call())
   )
 
   invisible(newRLumResults.fit_LMCurve)
